from typing import Tuple


class MyNode:
    def __init__(self, elem: int, node_left: 'MyNode' = None, node_right: 'MyNode' = None) -> None:
        self.elem = elem
        self.left = node_left
        self.right = node_right


class MyBST:
    def __init__(self) -> None:
        """creates an empty binary tree
        I only has an attribute: _root"""
        self._root = None

    def add(self, elem: int) -> None:
        if self._root is None:
            self._root = MyNode(elem)  # if tree is empty, new node will be the root
            return  # we can leave!!!

        node = self._root  # to search the place
        not_exist = True
        while not_exist and node:
            if elem < node.elem:
                if node.left is None:  # this is the place to insert it
                    node.left = MyNode(elem)
                    not_exist = False
                else:
                    node = node.left
            elif elem > node.elem:
                if node.right is None:  # this is the place to insert it
                    node.right = MyNode(elem)
                    not_exist = False
                else:
                    node = node.right
            elif elem == node.elem:
                print('duplicate elements not allowed!!', elem, node.elem)
                not_exist = False

    def draw(self) -> None:
        """function to draw a tree. """
        if self._root:
            self._draw('', self._root, False)
        else:
            print('tree is empty')
        print('\n\n')

    def _draw(self, prefix: str, node: MyNode, is_left: bool) -> None:
        if node is not None:
            self._draw(prefix + "     ", node.right, False)
            print(prefix + "|-- " + str(node.elem))
            self._draw(prefix + "     ", node.left, True)

    def inorder_range(self, start: int, end: int) -> list:
        """returns a sorted list with the elements in
         the range [start, end]"""
        result = []
        if start > end:
            return result
        self._inorder_range(self._root, start, end, result)
        return result

    def _inorder_range(self, node: MyNode, start: int, end: int, elems: list) -> None:
        """makes a sorted list with the elements in
         the range [start, end] in the subtree node"""
        if node:
            if node.elem > start:
                self._inorder_range(node.left, start, end, elems)
            if start <= node.elem <= end:
                elems.append(node.elem)
            if node.elem < end:
                self._inorder_range(node.right, start, end, elems)

    def get_average_range(self, start: int, end: int) -> float:
        """returns the average of the elements that are
        between start and end. O(n)"""
        # result = None
        result = 0 # None
        all_items = self.inorder_range(start, end)
        n = len(all_items)
        if n > 0:
            result = sum(all_items) / len(all_items)

        return result


if __name__ == "__main__":
    aux = MyBST()
    for x in [46, 11, 81, 51, 56, 94, 5, 20]:
        aux.add(x)

    aux.draw()
    pos1, pos2 = 5, 80

    print('get_average_range({}, {}) = {}'.format(pos1, pos2, aux.get_average_range(pos1, pos2)))
